@@ -101,6 +101,10 @@
 #ifndef HAVE_W32_SYSTEM
 #include <sys/utsname.h>
 #endif
+#ifdef MKDIR_TAKES_ONE_ARG
+#undef mkdir
+#define mkdir(a,b) mkdir(a)
+#endif
 
 #include "dirmngr.h"
 #include "validate.h"
@@ -125,13 +129,6 @@
 # define O_BINARY 0
 #endif
 
-
-/* Reason flags for an invalid CRL.  */
-#define INVCRL_TOO_OLD       1
-#define INVCRL_UNKNOWN_EXTN  2
-#define INVCRL_GENERAL       127
-
-
 static const char oidstr_crlNumber[] = "2.5.29.20";
 /* static const char oidstr_issuingDistributionPoint[] = "2.5.29.28"; */
 static const char oidstr_authorityKeyIdentifier[] = "2.5.29.35";
@@ -164,7 +161,7 @@ struct crl_cache_entry_s
   unsigned int cdb_use_count;  /* Current use count. */
   unsigned int cdb_lru_count;  /* Used for LRU purposes. */
   int dbfile_checked;          /* Set to true if the dbfile_hash value has
-                                  been checked once. */
+                                  been checked one. */
 };
 
 
@@ -209,15 +206,15 @@ get_current_cache (void)
 static int
 create_directory_if_needed (const char *name)
 {
-  gnupg_dir_t dir;
+  DIR *dir;
   char *fname;
 
   fname = make_filename (opt.homedir_cache, name, NULL);
-  dir = gnupg_opendir (fname);
+  dir = opendir (fname);
   if (!dir)
     {
       log_info (_("creating directory '%s'\n"), fname);
-      if (gnupg_mkdir (fname, "-rwx"))
+      if (mkdir (fname, S_IRUSR|S_IWUSR|S_IXUSR) )
         {
           int save_errno = errno;
           log_error (_("error creating directory '%s': %s\n"),
@@ -228,7 +225,7 @@ create_directory_if_needed (const char *name)
         }
     }
   else
-    gnupg_closedir (dir);
+    closedir (dir);
   xfree (fname);
   return 0;
 }
@@ -240,8 +237,8 @@ static int
 cleanup_cache_dir (int force)
 {
   char *dname = make_filename (opt.homedir_cache, DBDIR_D, NULL);
-  gnupg_dir_t dir;
-  gnupg_dirent_t de;
+  DIR *dir;
+  struct dirent *de;
   int problem = 0;
 
   if (!force)
@@ -254,7 +251,7 @@ cleanup_cache_dir (int force)
         }
     }
 
-  dir = gnupg_opendir (dname);
+  dir = opendir (dname);
   if (!dir)
     {
       log_error (_("error reading directory '%s': %s\n"),
@@ -263,7 +260,7 @@ cleanup_cache_dir (int force)
       return -1;
     }
 
-  while ((de = gnupg_readdir (dir)))
+  while ((de = readdir (dir)))
     {
       if (strcmp (de->d_name, "." ) && strcmp (de->d_name, ".."))
         {
@@ -274,7 +271,7 @@ cleanup_cache_dir (int force)
           if (force)
             okay = 1;
           else
-            okay = (!gnupg_stat (cdbname, &sbuf) && S_ISREG (sbuf.st_mode));
+            okay = (!stat (cdbname, &sbuf) && S_ISREG (sbuf.st_mode));
 
           if (okay)
             {
@@ -292,7 +289,7 @@ cleanup_cache_dir (int force)
         }
     }
   xfree (dname);
-  gnupg_closedir (dir);
+  closedir (dir);
   return problem;
 }
 
@@ -576,8 +573,8 @@ open_dir (crl_cache_t *r_cache)
           if (*line == 'i')
             {
               entry->invalid = atoi (line+1);
-              if (!entry->invalid)
-                entry->invalid = INVCRL_GENERAL;
+              if (entry->invalid < 1)
+                entry->invalid = 1;
             }
           else if (*line == 'u')
             entry->user_trust_req = 1;
@@ -991,8 +988,8 @@ make_db_file_name (const char *issuer_hash)
 
 
 /* Hash the file FNAME and return the MD5 digest in MD5BUFFER. The
- * caller must allocate MD5buffer with at least 16 bytes. Returns 0
- * on success. */
+   caller must allocate MD%buffer wityh at least 16 bytes. Returns 0
+   on success. */
 static int
 hash_dbfile (const char *fname, unsigned char *md5buffer)
 {
@@ -1145,7 +1142,7 @@ lock_db_file (crl_cache_t cache, crl_cache_entry_t entry)
       xfree (fname);
       return NULL;
     }
-  fd = gnupg_open (fname, O_RDONLY | O_BINARY, 0);
+  fd = open (fname, O_RDONLY | O_BINARY);
   if (fd == -1)
     {
       log_error (_("error opening cache file '%s': %s\n"),
@@ -1253,15 +1250,13 @@ crl_cache_deinit (void)
 }
 
 
-/* Delete the cache from disk and memory. Return 0 on success.*/
+/* Delete the cache from disk. Return 0 on success.*/
 int
 crl_cache_flush (void)
 {
   int rc;
 
-  crl_cache_deinit ();
   rc = cleanup_cache_dir (0)? -1 : 0;
-  crl_cache_init ();
 
   return rc;
 }
@@ -1354,7 +1349,7 @@ cache_isvalid (ctrl_t ctrl, const char *issuer_hash,
         {
           log_error (_("WARNING: invalid cache record length for S/N "));
           log_printf ("0x");
-          log_printhex (sn, snlen, "");
+          log_printhex ("", sn, snlen);
         }
       else if (opt.verbose)
         {
@@ -1402,7 +1397,7 @@ cache_isvalid (ctrl_t ctrl, const char *issuer_hash,
         {
           if (opt.verbose)
             log_info ("no system trust and client does not trust either\n");
-          retval = CRL_CACHE_NOTTRUSTED;
+          retval = CRL_CACHE_CANTUSE;
         }
       else
         {
@@ -1522,11 +1517,8 @@ crl_cache_cert_isvalid (ctrl_t ctrl, ksba_cert_t cert,
     case CRL_CACHE_DONTKNOW:
       err = gpg_error (GPG_ERR_NO_CRL_KNOWN);
       break;
-    case CRL_CACHE_NOTTRUSTED:
-      err = gpg_error (GPG_ERR_NOT_TRUSTED);
-      break;
     case CRL_CACHE_CANTUSE:
-      err = gpg_error (GPG_ERR_INV_CRL_OBJ);
+      err = gpg_error (GPG_ERR_NO_CRL_KNOWN);
       break;
     default:
       log_fatal ("cache_isvalid returned invalid status code %d\n", result);
@@ -1537,104 +1529,17 @@ crl_cache_cert_isvalid (ctrl_t ctrl, ksba_cert_t cert,
 }
 
 
-/* Return the hash algorithm's algo id from its name given in the
- * non-null termnated string in (buffer,buflen).  Returns 0 on failure
- * or if the algo is not known.  */
-static int
-hash_algo_from_buffer (const void *buffer, size_t buflen)
-{
-  char *string;
-  int algo;
-
-  string = xtrymalloc (buflen + 1);
-  if (!string)
-    {
-      log_error (_("out of core\n"));
-      return 0;
-    }
-  memcpy (string, buffer, buflen);
-  string[buflen] = 0;
-  algo = gcry_md_map_name (string);
-  if (!algo)
-    log_error ("unknown digest algorithm '%s' used in certificate\n", string);
-  xfree (string);
-  return algo;
-}
-
-
-/* Return an unsigned integer from the non-null termnated string
- * (buffer,buflen).  Returns 0 on failure.  */
-static unsigned int
-uint_from_buffer (const void *buffer, size_t buflen)
-{
-  char *string;
-  unsigned int val;
-
-  string = xtrymalloc (buflen + 1);
-  if (!string)
-    {
-      log_error (_("out of core\n"));
-      return 0;
-    }
-  memcpy (string, buffer, buflen);
-  string[buflen] = 0;
-  val = strtoul (string, NULL, 10);
-  xfree (string);
-  return val;
-}
-
-
 /* Prepare a hash context for the signature verification.  Input is
    the CRL and the output is the hash context MD as well as the uses
    algorithm identifier ALGO. */
 static gpg_error_t
-start_sig_check (ksba_crl_t crl, gcry_md_hd_t *md, int *algo, int *use_pss)
+start_sig_check (ksba_crl_t crl, gcry_md_hd_t *md, int *algo)
 {
   gpg_error_t err;
   const char *algoid;
 
-  *use_pss = 0;
   algoid = ksba_crl_get_digest_algo (crl);
-  if (algoid && !strcmp (algoid, "1.2.840.113549.1.1.10"))
-    {
-      /* Parse rsaPSS parameter.  */
-      gcry_buffer_t ioarray[1] = { {0} };
-      ksba_sexp_t pssparam;
-      size_t n;
-      gcry_sexp_t psssexp;
-
-      pssparam = ksba_crl_get_sig_val (crl);
-      n = gcry_sexp_canon_len (pssparam, 0, NULL, NULL);
-      if (!n)
-        {
-          ksba_free (pssparam);
-          log_error (_("got an invalid S-expression from libksba\n"));
-          return gpg_error (GPG_ERR_INV_SEXP);
-        }
-      err = gcry_sexp_sscan (&psssexp, NULL, pssparam, n);
-      ksba_free (pssparam);
-      if (err)
-        {
-          log_error (_("converting S-expression failed: %s\n"),
-                     gcry_strerror (err));
-          return err;
-        }
-
-      err = gcry_sexp_extract_param (psssexp, "sig-val",
-                                    "&'hash-algo'", ioarray, NULL);
-      gcry_sexp_release (psssexp);
-      if (err)
-        {
-          log_error ("extracting params from PSS failed: %s\n",
-                     gpg_strerror (err));
-          return err;
-        }
-      *algo = hash_algo_from_buffer (ioarray[0].data, ioarray[0].len);
-      xfree (ioarray[0].data);
-      *use_pss = 1;
-    }
-  else
-    *algo = gcry_md_map_name (algoid);
+  *algo = gcry_md_map_name (algoid);
   if (!*algo)
     {
       log_error (_("unknown hash algorithm '%s'\n"), algoid? algoid:"?");
@@ -1663,14 +1568,15 @@ start_sig_check (ksba_crl_t crl, gcry_md_hd_t *md, int *algo, int *use_pss)
    certificate of the CRL issuer.  This function takes ownership of MD.  */
 static gpg_error_t
 finish_sig_check (ksba_crl_t crl, gcry_md_hd_t md, int algo,
-                  ksba_cert_t issuer_cert, int use_pss)
+                  ksba_cert_t issuer_cert)
 {
   gpg_error_t err;
   ksba_sexp_t sigval = NULL, pubkey = NULL;
+  const char *s;
+  char algoname[50];
   size_t n;
   gcry_sexp_t s_sig = NULL, s_hash = NULL, s_pkey = NULL;
-  unsigned int saltlen = 0;  /* (used only with use_pss)  */
-  int pkalgo;
+  unsigned int i;
 
   /* This also stops debugging on the MD.  */
   gcry_md_final (md);
@@ -1692,81 +1598,6 @@ finish_sig_check (ksba_crl_t crl, gcry_md_hd_t md, int algo,
       goto leave;
     }
 
-  if (use_pss)
-    {
-      /* Parse rsaPSS parameter which we should find in S_SIG.  */
-      gcry_buffer_t ioarray[2] = { {0}, {0} };
-      ksba_sexp_t pssparam;
-      gcry_sexp_t psssexp;
-      int hashalgo;
-
-      pssparam = ksba_crl_get_sig_val (crl);
-      n = gcry_sexp_canon_len (pssparam, 0, NULL, NULL);
-      if (!n)
-        {
-          ksba_free (pssparam);
-          log_error (_("got an invalid S-expression from libksba\n"));
-          err = gpg_error (GPG_ERR_INV_SEXP);
-          goto leave;
-        }
-      err = gcry_sexp_sscan (&psssexp, NULL, pssparam, n);
-      ksba_free (pssparam);
-      if (err)
-        {
-          log_error (_("converting S-expression failed: %s\n"),
-                     gcry_strerror (err));
-          goto leave;
-        }
-
-      err = gcry_sexp_extract_param (psssexp, "sig-val",
-                                    "&'hash-algo''salt-length'",
-                                     ioarray+0, ioarray+1, NULL);
-      gcry_sexp_release (psssexp);
-      if (err)
-        {
-          log_error ("extracting params from PSS failed: %s\n",
-                     gpg_strerror (err));
-          goto leave;
-        }
-      hashalgo = hash_algo_from_buffer (ioarray[0].data, ioarray[0].len);
-      saltlen = uint_from_buffer (ioarray[1].data, ioarray[1].len);
-      xfree (ioarray[0].data);
-      xfree (ioarray[1].data);
-      if (hashalgo != algo)
-        {
-          log_error ("hash algo mismatch: %d announced but %d used\n",
-                     algo, hashalgo);
-          err = gpg_error (GPG_ERR_INV_CRL);
-          goto leave;
-        }
-      /* Add some restrictions; see ../sm/certcheck.c for details.  */
-      switch (algo)
-        {
-        case GCRY_MD_SHA1:
-        case GCRY_MD_SHA256:
-        case GCRY_MD_SHA384:
-        case GCRY_MD_SHA512:
-        case GCRY_MD_SHA3_256:
-        case GCRY_MD_SHA3_384:
-        case GCRY_MD_SHA3_512:
-          break;
-        default:
-          log_error ("PSS hash algorithm '%s' rejected\n",
-                     gcry_md_algo_name (algo));
-          err = gpg_error (GPG_ERR_DIGEST_ALGO);
-          goto leave;
-        }
-
-      if (gcry_md_get_algo_dlen (algo) != saltlen)
-        {
-          log_error ("PSS hash algorithm '%s' rejected due to salt length %u\n",
-                     gcry_md_algo_name (algo), saltlen);
-          err = gpg_error (GPG_ERR_DIGEST_ALGO);
-          goto leave;
-        }
-    }
-
-
   /* Get and convert the public key for the issuer certificate. */
   if (DBG_X509)
     dump_cert ("crl_issuer_cert", issuer_cert);
@@ -1787,73 +1618,13 @@ finish_sig_check (ksba_crl_t crl, gcry_md_hd_t md, int algo,
     }
 
   /* Create an S-expression with the actual hash value. */
-  if (use_pss)
-    {
-      err = gcry_sexp_build (&s_hash, NULL,
-                             "(data (flags pss)"
-                             "(hash %s %b)"
-                             "(salt-length %u))",
-                             hash_algo_to_string (algo),
-                             (int)gcry_md_get_algo_dlen (algo),
-                             gcry_md_read (md, algo),
-                             saltlen);
-    }
-  else if ((pkalgo = pk_algo_from_sexp (s_pkey)) == GCRY_PK_ECC)
-    {
-      unsigned int qbits0, qbits;
-
-      qbits0 = gcry_pk_get_nbits (s_pkey);
-      qbits = qbits0 == 521? 512 : qbits0;
-
-      if ((qbits%8))
-	{
-	  log_error ("ECDSA requires the hash length to be a"
-                     " multiple of 8 bits\n");
-	  err = gpg_error (GPG_ERR_INTERNAL);
-          goto leave;
-	}
-
-      /* Don't allow any Q smaller than 160 bits.  */
-      if (qbits < 160)
-	{
-	  log_error (_("%s key uses an unsafe (%u bit) hash\n"),
-                     gcry_pk_algo_name (pkalgo), qbits0);
-	  err = gpg_error (GPG_ERR_INTERNAL);
-          goto leave;
-	}
-
-      /* Check if we're too short.  */
-      n = gcry_md_get_algo_dlen (algo);
-      if (n < qbits/8)
-        {
-	  log_error (_("a %u bit hash is not valid for a %u bit %s key\n"),
-                     (unsigned int)n*8,
-                     qbits0,
-                     gcry_pk_algo_name (pkalgo));
-          if (n < 20)
-            {
-              err = gpg_error (GPG_ERR_INTERNAL);
-              goto leave;
-            }
-        }
-
-      /* Truncate.  */
-      if (n > qbits/8)
-        n = qbits/8;
-
-      err = gcry_sexp_build (&s_hash, NULL, "(data(flags raw)(value %b))",
-                             (int)n,
-                             gcry_md_read (md, algo));
-
-    }
-  else
-    {
-      err = gcry_sexp_build (&s_hash, NULL,
-                             "(data(flags pkcs1)(hash %s %b))",
-                             hash_algo_to_string (algo),
-                             (int)gcry_md_get_algo_dlen (algo),
-                             gcry_md_read (md, algo));
-    }
+  s = gcry_md_algo_name (algo);
+  for (i = 0; *s && i < sizeof(algoname) - 1; s++, i++)
+    algoname[i] = ascii_tolower (*s);
+  algoname[i] = 0;
+  err = gcry_sexp_build (&s_hash, NULL, "(data(flags pkcs1)(hash %s %b))",
+                         algoname,
+                         gcry_md_get_algo_dlen (algo), gcry_md_read (md, algo));
   if (err)
     {
       log_error (_("creating S-expression failed: %s\n"), gcry_strerror (err));
@@ -1863,7 +1634,7 @@ finish_sig_check (ksba_crl_t crl, gcry_md_hd_t md, int algo,
   /* Pass this on to the signature verification. */
   err = gcry_pk_verify (s_sig, s_hash, s_pkey);
   if (DBG_X509)
-    log_debug ("%s: gcry_pk_verify: %s\n", __func__, gpg_strerror (err));
+    log_debug ("gcry_pk_verify: %s\n", gpg_strerror (err));
 
  leave:
   xfree (sigval);
@@ -1915,7 +1686,6 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
   ksba_cert_t crlissuer_cert = NULL;
   gcry_md_hd_t md = NULL;
   int algo = 0;
-  int use_pss = 0;
   size_t n;
 
   (void)fname;
@@ -1938,7 +1708,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
         {
         case KSBA_SR_BEGIN_ITEMS:
           {
-            err = start_sig_check (crl, &md, &algo, &use_pss);
+            err = start_sig_check (crl, &md, &algo);
             if (err)
               goto failure;
 
@@ -2012,7 +1782,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
             ksba_sexp_t keyid;
 
             /* We need to look for the issuer only after having read
-               all items.  The issuer itself comes before the items
+               all items.  The issuer itselfs comes before the items
                but the optional authorityKeyIdentifier comes after the
                items. */
             err = ksba_crl_get_issuer (crl, &crlissuer);
@@ -2075,7 +1845,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
                 goto failure;
               }
 
-            err = finish_sig_check (crl, md, algo, crlissuer_cert, use_pss);
+            err = finish_sig_check (crl, md, algo, crlissuer_cert);
             md = NULL; /* Closed.  */
             if (err)
               {
@@ -2086,7 +1856,6 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
 
             err = validate_cert_chain (ctrl, crlissuer_cert, NULL,
                                        (VALIDATE_FLAG_TRUST_CONFIG
-                                        | VALIDATE_FLAG_TRUST_SYSTEM
                                         | VALIDATE_FLAG_CRL
                                         | VALIDATE_FLAG_RECURSIVE),
                                        r_trust_anchor);
@@ -2108,7 +1877,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
         }
     }
   while (stopreason != KSBA_SR_READY);
-  log_assert (!err);
+  assert (!err);
 
 
  failure:
@@ -2138,7 +1907,7 @@ get_crl_number (ksba_crl_t crl)
 
 
 /* Return the authorityKeyIdentifier or NULL if it is not available.
-   The issuer name may consists of several parts - they are delimited by
+   The issuer name may consists of several parts - they are delimted by
    0x01. */
 static char *
 get_auth_key_id (ksba_crl_t crl, char **serialno)
@@ -2285,7 +2054,7 @@ crl_cache_insert (ctrl_t ctrl, const char *url, ksba_reader_t reader)
       }
   }
 
-  fd_cdb = gnupg_open (fname, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644);
+  fd_cdb = open (fname, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644);
   if (fd_cdb == -1)
     {
       err = gpg_error_from_errno (errno);
@@ -2349,7 +2118,7 @@ crl_cache_insert (ctrl_t ctrl, const char *url, ksba_reader_t reader)
                      nextupdate);
           if (!err2)
             err2 = gpg_error (GPG_ERR_CRL_TOO_OLD);
-          invalidate_crl |= INVCRL_TOO_OLD;
+          invalidate_crl |= 1;
         }
     }
 
@@ -2357,24 +2126,14 @@ crl_cache_insert (ctrl_t ctrl, const char *url, ksba_reader_t reader)
   for (idx=0; !(err=ksba_crl_get_extension (crl, idx, &oid, &critical,
                                               NULL, NULL)); idx++)
     {
-      strlist_t sl;
-
       if (!critical
           || !strcmp (oid, oidstr_authorityKeyIdentifier)
           || !strcmp (oid, oidstr_crlNumber) )
         continue;
-
-      for (sl=opt.ignored_crl_extensions;
-           sl && strcmp (sl->d, oid); sl = sl->next)
-        ;
-      if (sl)
-        continue;  /* Is in ignored list.  */
-
       log_error (_("unknown critical CRL extension %s\n"), oid);
-      log_info ("(CRL='%s')\n", url);
       if (!err2)
         err2 = gpg_error (GPG_ERR_INV_CRL);
-      invalidate_crl |= INVCRL_UNKNOWN_EXTN;
+      invalidate_crl |= 2;
     }
   if (gpg_err_code (err) == GPG_ERR_EOF
       || gpg_err_code (err) == GPG_ERR_NO_DATA )
@@ -2513,7 +2272,6 @@ list_one_crl_entry (crl_cache_t cache, crl_cache_entry_t e, estream_t fp)
   int rc;
   int warn = 0;
   const unsigned char *s;
-  unsigned int invalid;
 
   es_fputs ("--------------------------------------------------------\n", fp );
   es_fprintf (fp, _("Begin CRL dump (retrieved via %s)\n"), e->url );
@@ -2538,20 +2296,13 @@ list_one_crl_entry (crl_cache_t cache, crl_cache_entry_t e, estream_t fp)
               !e->user_trust_req? "[system]" :
               e->check_trust_anchor? e->check_trust_anchor:"[missing]");
 
-  invalid = e->invalid;
-  if ((invalid & INVCRL_TOO_OLD))
-    {
-      invalid &= ~INVCRL_TOO_OLD;
-      es_fprintf (fp, _(" ERROR: The CRL will not be used "
-                        "because it was still too old after an update!\n"));
-    }
-  if ((invalid & INVCRL_UNKNOWN_EXTN))
-    {
-      invalid &= ~INVCRL_UNKNOWN_EXTN;
-      es_fprintf (fp, _(" ERROR: The CRL will not be used "
+  if ((e->invalid & 1))
+    es_fprintf (fp, _(" ERROR: The CRL will not be used "
+                      "because it was still too old after an update!\n"));
+  if ((e->invalid & 2))
+    es_fprintf (fp, _(" ERROR: The CRL will not be used "
                       "due to an unknown critical extension!\n"));
-    }
-  if (invalid)  /* INVCRL_GENERAL or some other bits are set.  */
+  if ((e->invalid & ~3))
     es_fprintf (fp, _(" ERROR: The CRL will not be used\n"));
 
   cdb = lock_db_file (cache, e);
@@ -2692,21 +2443,21 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
   ksba_name_t distpoint = NULL;
   ksba_name_t issuername = NULL;
   char *distpoint_uri = NULL;
+  char *issuername_uri = NULL;
   int any_dist_point = 0;
   int seq;
-  gpg_error_t last_err = 0;
 
   /* Loop over all distribution points, get the CRLs and put them into
      the cache. */
   if (opt.verbose)
     log_info ("checking distribution points\n");
   seq = 0;
-  while (xfree (distpoint), xfree (issuername),
-         !(err = ksba_cert_get_crl_dist_point (cert, seq++,
+  while ( !(err = ksba_cert_get_crl_dist_point (cert, seq++,
                                                 &distpoint,
                                                 &issuername, NULL )))
     {
       int name_seq;
+      gpg_error_t last_err = 0;
 
       if (!distpoint && !issuername)
         {
@@ -2717,11 +2468,13 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
                     suitable CRL. */
         }
 
+      xfree (issuername_uri); issuername_uri = NULL;
+
       /* Get the URIs.  We do this in a loop to iterate over all names
          in the crlDP. */
       for (name_seq=0; ksba_name_enum (distpoint, name_seq); name_seq++)
         {
-          xfree (distpoint_uri);
+          xfree (distpoint_uri); distpoint_uri = NULL;
           distpoint_uri = ksba_name_get_uri (distpoint, name_seq);
           if (!distpoint_uri)
             continue;
@@ -2743,7 +2496,8 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
 
           any_dist_point = 1;
 
-          crl_close_reader (reader);
+          if (opt.verbose)
+            log_info ("fetching CRL from '%s'\n", distpoint_uri);
           err = crl_fetch (ctrl, distpoint_uri, &reader);
           if (err)
             {
@@ -2763,16 +2517,28 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
               last_err = err;
               continue; /* with the next name. */
             }
-          goto leave; /* Ready - we got the CRL. */
+          last_err = 0;
+          break; /* Ready. */
         }
+      if (last_err)
+        {
+          err = last_err;
+          goto leave;
+        }
+
+      ksba_name_release (distpoint); distpoint = NULL;
+
+      /* We don't do anything with issuername_uri yet but we keep the
+         code for documentation. */
+      issuername_uri =  ksba_name_get_uri (issuername, 0);
+      ksba_name_release (issuername); issuername = NULL;
+
+      /* Close the reader.  */
+      crl_close_reader (reader);
+      reader = NULL;
     }
   if (gpg_err_code (err) == GPG_ERR_EOF)
     err = 0;
-  if (!err && last_err)
-    {
-      err = last_err;
-      goto leave;
-    }
 
   /* If we did not found any distpoint, try something reasonable. */
   if (!any_dist_point )
@@ -2780,6 +2546,9 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
       if (opt.verbose)
         log_info ("no distribution point - trying issuer name\n");
 
+      crl_close_reader (reader);
+      reader = NULL;
+
       issuer = ksba_cert_get_issuer (cert, 0);
       if (!issuer)
         {
@@ -2790,7 +2559,6 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
 
       if (opt.verbose)
         log_info ("fetching CRL from default location\n");
-      crl_close_reader (reader);
       err = crl_fetch_default (ctrl, issuer, &reader);
       if (err)
           {
@@ -2813,6 +2581,7 @@ crl_cache_reload_crl (ctrl_t ctrl, ksba_cert_t cert)
  leave:
   crl_close_reader (reader);
   xfree (distpoint_uri);
+  xfree (issuername_uri);
   ksba_name_release (distpoint);
   ksba_name_release (issuername);
   ksba_free (issuer);
