@@ -38,7 +38,7 @@
 #include "../common/compliance.h"
 
 
-static gpg_error_t get_it (ctrl_t ctrl, struct pubkey_enc_list *k,
+static gpg_error_t get_it (ctrl_t ctrl, PKT_pubkey_enc *k,
                            DEK *dek, PKT_public_key *sk, u32 *keyid);
 
 
@@ -72,124 +72,101 @@ is_algo_in_prefs (kbnode_t keyblock, preftype_t type, int algo)
  * which should have been allocated in secure memory by the caller.
  */
 gpg_error_t
-get_session_key (ctrl_t ctrl, struct pubkey_enc_list *list, DEK *dek)
+get_session_key (ctrl_t ctrl, PKT_pubkey_enc * k, DEK * dek)
 {
   PKT_public_key *sk = NULL;
-  gpg_error_t err;
-  void *enum_context = NULL;
-  u32 keyid[2];
-  int search_for_secret_keys = 1;
-  struct pubkey_enc_list *k;
+  int rc;
 
   if (DBG_CLOCK)
     log_clock ("get_session_key enter");
 
-  while (search_for_secret_keys)
+  rc = openpgp_pk_test_algo2 (k->pubkey_algo, PUBKEY_USAGE_ENC);
+  if (rc)
+    goto leave;
+
+  if ((k->keyid[0] || k->keyid[1]) && !opt.try_all_secrets)
     {
       sk = xmalloc_clear (sizeof *sk);
-      err = enum_secret_keys (ctrl, &enum_context, sk);
-      if (err)
-        break;
-
-      /* Check compliance.  */
-      if (! gnupg_pk_is_allowed (opt.compliance, PK_USE_DECRYPTION,
-                                 sk->pubkey_algo, 0,
-                                 sk->pkey, nbits_from_pk (sk), NULL))
+      sk->pubkey_algo = k->pubkey_algo; /* We want a pubkey with this algo.  */
+      if (!(rc = get_seckey (ctrl, sk, k->keyid)))
         {
-          log_info (_("key %s is not suitable for decryption"
-                      " in %s mode\n"),
-                    keystr_from_pk (sk),
-                    gnupg_compliance_option_string (opt.compliance));
-          continue;
-        }
-
-      /* FIXME: The list needs to be sorted so that we try the keys in
-       * an appropriate order.  For example:
-       * - On-disk keys w/o protection
-       * - On-disk keys with a cached passphrase
-       * - On-card keys of an active card
-       * - On-disk keys with protection
-       * - On-card keys from cards which are not plugged it.  Here a
-       *   cancel-all button should stop asking for other cards.
-       * Without any anonymous keys the sorting can be skipped.
-       */
-      for (k = list; k; k = k->next)
-        {
-          if (!(k->pubkey_algo == PUBKEY_ALGO_ELGAMAL_E
-                || k->pubkey_algo == PUBKEY_ALGO_ECDH
-                || k->pubkey_algo == PUBKEY_ALGO_RSA
-                || k->pubkey_algo == PUBKEY_ALGO_RSA_E
-                || k->pubkey_algo == PUBKEY_ALGO_ELGAMAL))
-            continue;
-
-          if (openpgp_pk_test_algo2 (k->pubkey_algo, PUBKEY_USAGE_ENC))
-            continue;
-
-          if (sk->pubkey_algo != k->pubkey_algo)
-            continue;
-
-          keyid_from_pk (sk, keyid);
-
-          if (!k->keyid[0] && !k->keyid[1])
+          /* Check compliance.  */
+          if (! gnupg_pk_is_allowed (opt.compliance, PK_USE_DECRYPTION,
+                                     sk->pubkey_algo,
+                                     sk->pkey, nbits_from_pk (sk), NULL))
             {
-              if (opt.skip_hidden_recipients)
-                continue;
-
-              if (!opt.quiet)
-                log_info (_("anonymous recipient; trying secret key %s ...\n"),
-                          keystr (keyid));
-            }
-          else if (opt.try_all_secrets
-                   || (k->keyid[0] == keyid[0] && k->keyid[1] == keyid[1]))
-            {
-              if (!opt.quiet && !(sk->pubkey_usage & PUBKEY_USAGE_XENC_MASK))
-                log_info (_("used key is not marked for encryption use.\n"));
+              log_info (_("key %s is not suitable for decryption"
+                          " in %s mode\n"),
+                        keystr_from_pk (sk),
+                        gnupg_compliance_option_string (opt.compliance));
+              rc = gpg_error (GPG_ERR_PUBKEY_ALGO);
             }
           else
-            continue;
-
-          err = get_it (ctrl, k, dek, sk, keyid);
-          k->result = err;
-          if (!err)
-            {
-              if (!opt.quiet && !k->keyid[0] && !k->keyid[1])
-                {
-                  log_info (_("okay, we are the anonymous recipient.\n"));
-                  if (!(sk->pubkey_usage & PUBKEY_USAGE_XENC_MASK))
-                    log_info (_("used key is not marked for encryption use.\n")
-                              );
-                }
-              search_for_secret_keys = 0;
-              break;
-            }
-          else if (gpg_err_code (err) == GPG_ERR_FULLY_CANCELED)
-            {
-              search_for_secret_keys = 0;
-              break; /* Don't try any more secret keys.  */
-            }
+            rc = get_it (ctrl, k, dek, sk, k->keyid);
         }
     }
-  enum_secret_keys (ctrl, &enum_context, NULL);  /* free context */
-
-  if (gpg_err_code (err) == GPG_ERR_EOF)
+  else if (opt.skip_hidden_recipients)
+    rc = gpg_error (GPG_ERR_NO_SECKEY);
+  else  /* Anonymous receiver: Try all available secret keys.  */
     {
-      err = gpg_error (GPG_ERR_NO_SECKEY);
+      void *enum_context = NULL;
+      u32 keyid[2];
 
-      /* Return the last specific error, if any.  */
-      for (k = list; k; k = k->next)
-        if (k->result != -1)
-          err = k->result;
+      for (;;)
+        {
+          free_public_key (sk);
+          sk = xmalloc_clear (sizeof *sk);
+          rc = enum_secret_keys (ctrl, &enum_context, sk);
+          if (rc)
+            {
+              rc = GPG_ERR_NO_SECKEY;
+              break;
+            }
+          if (sk->pubkey_algo != k->pubkey_algo)
+            continue;
+          if (!(sk->pubkey_usage & PUBKEY_USAGE_ENC))
+            continue;
+          keyid_from_pk (sk, keyid);
+          if (!opt.quiet)
+            log_info (_("anonymous recipient; trying secret key %s ...\n"),
+                      keystr (keyid));
+
+          /* Check compliance.  */
+          if (! gnupg_pk_is_allowed (opt.compliance, PK_USE_DECRYPTION,
+                                     sk->pubkey_algo,
+                                     sk->pkey, nbits_from_pk (sk), NULL))
+            {
+              log_info (_("key %s is not suitable for decryption"
+                          " in %s mode\n"),
+                          keystr_from_pk (sk),
+                          gnupg_compliance_option_string (opt.compliance));
+              continue;
+            }
+
+          rc = get_it (ctrl, k, dek, sk, keyid);
+          if (!rc)
+            {
+              if (!opt.quiet)
+                log_info (_("okay, we are the anonymous recipient.\n"));
+              break;
+            }
+          else if (gpg_err_code (rc) == GPG_ERR_FULLY_CANCELED)
+            break; /* Don't try any more secret keys.  */
+        }
+      enum_secret_keys (ctrl, &enum_context, NULL);  /* free context */
     }
 
+ leave:
+  free_public_key (sk);
   if (DBG_CLOCK)
     log_clock ("get_session_key leave");
-  return err;
+  return rc;
 }
 
 
 static gpg_error_t
 get_it (ctrl_t ctrl,
-        struct pubkey_enc_list *enc, DEK *dek, PKT_public_key *sk, u32 *keyid)
+        PKT_pubkey_enc *enc, DEK *dek, PKT_public_key *sk, u32 *keyid)
 {
   gpg_error_t err;
   byte *frame = NULL;
@@ -201,6 +178,7 @@ get_it (ctrl_t ctrl,
   char *desc;
   char *keygrip;
   byte fp[MAX_FINGERPRINT_LEN];
+  size_t fpn;
 
   if (DBG_CLOCK)
     log_clock ("decryption start");
@@ -244,7 +222,10 @@ get_it (ctrl_t ctrl,
     goto leave;
 
   if (sk->pubkey_algo == PUBKEY_ALGO_ECDH)
-    fingerprint_from_pk (sk, fp, NULL);
+    {
+      fingerprint_from_pk (sk, fp, &fpn);
+      log_assert (fpn == 20);
+    }
 
   /* Decrypt. */
   desc = gpg_format_keydesc (ctrl, sk, FORMAT_KEYDESC_NORMAL, 1);
@@ -274,16 +255,25 @@ get_it (ctrl_t ctrl,
    * CSUM
    */
   if (DBG_CRYPTO)
-    log_printhex (frame, nframe, "DEK frame:");
+    log_printhex ("DEK frame:", frame, nframe);
   n = 0;
 
   if (sk->pubkey_algo == PUBKEY_ALGO_ECDH)
     {
+      gcry_mpi_t shared_mpi;
       gcry_mpi_t decoded;
 
       /* At the beginning the frame are the bytes of shared point MPI.  */
+      err = gcry_mpi_scan (&shared_mpi, GCRYMPI_FMT_USG, frame, nframe, NULL);
+      if (err)
+        {
+          err = gpg_error (GPG_ERR_WRONG_SECKEY);
+          goto leave;
+        }
+
       err = pk_ecdh_decrypt (&decoded, fp, enc->data[1]/*encr data as an MPI*/,
-                             frame, nframe, sk->pkey);
+                             shared_mpi, sk->pkey);
+      mpi_release (shared_mpi);
       if(err)
         goto leave;
 
@@ -294,7 +284,10 @@ get_it (ctrl_t ctrl,
         goto leave;
 
       /* Now the frame are the bytes decrypted but padded session key.  */
-      if (!nframe || nframe <= 8
+
+      /* Allow double padding for the benefit of DEK size concealment.
+         Higher than this is wasteful. */
+      if (!nframe || frame[nframe-1] > 8*2 || nframe <= 8
           || frame[nframe-1] > nframe)
         {
           err = gpg_error (GPG_ERR_WRONG_SECKEY);
@@ -312,16 +305,6 @@ get_it (ctrl_t ctrl,
               err = gpg_error (GPG_ERR_WRONG_SECKEY);
               goto leave;
             }
-
-          /* FIXME: Actually the leading zero is required but due to
-           * the way we encode the output in libgcrypt as an MPI we
-           * are not able to encode that leading zero.  However, when
-           * using a Smartcard we are doing it the right way and
-           * therefore we have to skip the zero.  This should be fixed
-           * in gpg-agent of course. */
-          if (!frame[n])
-            n++;
-
           if (frame[n] == 1 && frame[nframe - 1] == 2)
             {
               log_info (_("old encoding of the DEK is not supported\n"));
@@ -378,19 +361,19 @@ get_it (ctrl_t ctrl,
   if (DBG_CLOCK)
     log_clock ("decryption ready");
   if (DBG_CRYPTO)
-    log_printhex (dek->key, dek->keylen, "DEK is:");
+    log_printhex ("DEK is:", dek->key, dek->keylen);
 
   /* Check that the algo is in the preferences and whether it has
    * expired.  Also print a status line with the key's fingerprint.  */
   {
     PKT_public_key *pk = NULL;
     PKT_public_key *mainpk = NULL;
-    KBNODE pkb = get_pubkeyblock_ext (ctrl, keyid, GET_PUBKEYBLOCK_FLAG_ADSK);
+    KBNODE pkb = get_pubkeyblock (ctrl, keyid);
 
     if (!pkb)
       {
-        err = gpg_error (GPG_ERR_UNEXPECTED);
-        log_info ("oops: public key not found for preference check\n");
+        err = -1;
+        log_error ("oops: public key not found for preference check\n");
       }
     else if (pkb->pkt->pkt.public_key->selfsigversion > 3
              && dek->algo != CIPHER_ALGO_3DES
@@ -434,13 +417,6 @@ get_it (ctrl_t ctrl,
           }
       }
 
-    if (pk && !(pk->pubkey_usage & PUBKEY_USAGE_ENC)
-        && (pk->pubkey_usage & PUBKEY_USAGE_RENC))
-      {
-        log_info (_("Note: ADSK key has been used for decryption"));
-        log_printf ("\n");
-      }
-
     if (pk && pk->flags.revoked)
       {
         log_info (_("Note: key has been revoked"));
@@ -458,7 +434,7 @@ get_it (ctrl_t ctrl,
 
         /* Note that we do not want to create a trustdb just for
          * getting the ownertrust: If there is no trustdb there can't
-         * be an ultimately trusted key anyway and thus the ownertrust
+         * be ulitmately trusted key anyway and thus the ownertrust
          * value is irrelevant.  */
         write_status_printf (STATUS_DECRYPTION_KEY, "%s %s %c",
                              pkhex, mainpkhex,
